import typer

from savvihub.api.exceptions import NotFoundAPIException
from savvihub.cli.exceptions import ExitException
from savvihub.cli.typer import Context
from savvihub.cli.utils import get_default_organization


def organization_name_callback(ctx: Context, organization_name: str) -> str:
    if organization_name:
        try:
            ctx.authenticated_client.organization_read(organization_name)
        except NotFoundAPIException:
            raise ExitException('Organization not found.')
        return organization_name

    return (ctx.organization.name if ctx.organization
            else get_default_organization(ctx.authenticated_client).name)


organization_name_option = typer.Option(None, '--organization', callback=organization_name_callback,
                                        help='Override the default organization.')



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class GithubLicense(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'body': 'str',
        'conditions': 'list[str]',
        'description': 'str',
        'featured': 'bool',
        'html_url': 'str',
        'implementation': 'str',
        'key': 'str',
        'limitations': 'list[str]',
        'name': 'str',
        'permissions': 'list[str]',
        'spdx_id': 'str',
        'url': 'str'
    }

    attribute_map = {
        'body': 'body',
        'conditions': 'conditions',
        'description': 'description',
        'featured': 'featured',
        'html_url': 'html_url',
        'implementation': 'implementation',
        'key': 'key',
        'limitations': 'limitations',
        'name': 'name',
        'permissions': 'permissions',
        'spdx_id': 'spdx_id',
        'url': 'url'
    }

    def __init__(self, body=None, conditions=None, description=None, featured=None, html_url=None, implementation=None, key=None, limitations=None, name=None, permissions=None, spdx_id=None, url=None, local_vars_configuration=None):  # noqa: E501
        """GithubLicense - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._body = None
        self._conditions = None
        self._description = None
        self._featured = None
        self._html_url = None
        self._implementation = None
        self._key = None
        self._limitations = None
        self._name = None
        self._permissions = None
        self._spdx_id = None
        self._url = None
        self.discriminator = None

        self.body = body
        if conditions is not None:
            self.conditions = conditions
        self.description = description
        self.featured = featured
        self.html_url = html_url
        self.implementation = implementation
        self.key = key
        if limitations is not None:
            self.limitations = limitations
        self.name = name
        if permissions is not None:
            self.permissions = permissions
        self.spdx_id = spdx_id
        self.url = url

    @property
    def body(self):
        """Gets the body of this GithubLicense.  # noqa: E501


        :return: The body of this GithubLicense.  # noqa: E501
        :rtype: str
        """
        return self._body

    @body.setter
    def body(self, body):
        """Sets the body of this GithubLicense.


        :param body: The body of this GithubLicense.  # noqa: E501
        :type body: str
        """

        self._body = body

    @property
    def conditions(self):
        """Gets the conditions of this GithubLicense.  # noqa: E501


        :return: The conditions of this GithubLicense.  # noqa: E501
        :rtype: list[str]
        """
        return self._conditions

    @conditions.setter
    def conditions(self, conditions):
        """Sets the conditions of this GithubLicense.


        :param conditions: The conditions of this GithubLicense.  # noqa: E501
        :type conditions: list[str]
        """

        self._conditions = conditions

    @property
    def description(self):
        """Gets the description of this GithubLicense.  # noqa: E501


        :return: The description of this GithubLicense.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this GithubLicense.


        :param description: The description of this GithubLicense.  # noqa: E501
        :type description: str
        """

        self._description = description

    @property
    def featured(self):
        """Gets the featured of this GithubLicense.  # noqa: E501


        :return: The featured of this GithubLicense.  # noqa: E501
        :rtype: bool
        """
        return self._featured

    @featured.setter
    def featured(self, featured):
        """Sets the featured of this GithubLicense.


        :param featured: The featured of this GithubLicense.  # noqa: E501
        :type featured: bool
        """

        self._featured = featured

    @property
    def html_url(self):
        """Gets the html_url of this GithubLicense.  # noqa: E501


        :return: The html_url of this GithubLicense.  # noqa: E501
        :rtype: str
        """
        return self._html_url

    @html_url.setter
    def html_url(self, html_url):
        """Sets the html_url of this GithubLicense.


        :param html_url: The html_url of this GithubLicense.  # noqa: E501
        :type html_url: str
        """

        self._html_url = html_url

    @property
    def implementation(self):
        """Gets the implementation of this GithubLicense.  # noqa: E501


        :return: The implementation of this GithubLicense.  # noqa: E501
        :rtype: str
        """
        return self._implementation

    @implementation.setter
    def implementation(self, implementation):
        """Sets the implementation of this GithubLicense.


        :param implementation: The implementation of this GithubLicense.  # noqa: E501
        :type implementation: str
        """

        self._implementation = implementation

    @property
    def key(self):
        """Gets the key of this GithubLicense.  # noqa: E501


        :return: The key of this GithubLicense.  # noqa: E501
        :rtype: str
        """
        return self._key

    @key.setter
    def key(self, key):
        """Sets the key of this GithubLicense.


        :param key: The key of this GithubLicense.  # noqa: E501
        :type key: str
        """

        self._key = key

    @property
    def limitations(self):
        """Gets the limitations of this GithubLicense.  # noqa: E501


        :return: The limitations of this GithubLicense.  # noqa: E501
        :rtype: list[str]
        """
        return self._limitations

    @limitations.setter
    def limitations(self, limitations):
        """Sets the limitations of this GithubLicense.


        :param limitations: The limitations of this GithubLicense.  # noqa: E501
        :type limitations: list[str]
        """

        self._limitations = limitations

    @property
    def name(self):
        """Gets the name of this GithubLicense.  # noqa: E501


        :return: The name of this GithubLicense.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this GithubLicense.


        :param name: The name of this GithubLicense.  # noqa: E501
        :type name: str
        """

        self._name = name

    @property
    def permissions(self):
        """Gets the permissions of this GithubLicense.  # noqa: E501


        :return: The permissions of this GithubLicense.  # noqa: E501
        :rtype: list[str]
        """
        return self._permissions

    @permissions.setter
    def permissions(self, permissions):
        """Sets the permissions of this GithubLicense.


        :param permissions: The permissions of this GithubLicense.  # noqa: E501
        :type permissions: list[str]
        """

        self._permissions = permissions

    @property
    def spdx_id(self):
        """Gets the spdx_id of this GithubLicense.  # noqa: E501


        :return: The spdx_id of this GithubLicense.  # noqa: E501
        :rtype: str
        """
        return self._spdx_id

    @spdx_id.setter
    def spdx_id(self, spdx_id):
        """Sets the spdx_id of this GithubLicense.


        :param spdx_id: The spdx_id of this GithubLicense.  # noqa: E501
        :type spdx_id: str
        """

        self._spdx_id = spdx_id

    @property
    def url(self):
        """Gets the url of this GithubLicense.  # noqa: E501


        :return: The url of this GithubLicense.  # noqa: E501
        :rtype: str
        """
        return self._url

    @url.setter
    def url(self, url):
        """Sets the url of this GithubLicense.


        :param url: The url of this GithubLicense.  # noqa: E501
        :type url: str
        """

        self._url = url

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, GithubLicense):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, GithubLicense):
            return True

        return self.to_dict() != other.to_dict()



from datetime import datetime
from typing import List, Optional

import typer
from terminaltables import AsciiTable

from openapi_client import (
    ModelKernelResourceSpecField,
    ModelParameter,
    ModelRange,
    ModelSweepObjective,
    ModelSweepSearchSpace,
    ProtoVolumeMountRequest,
    ProtoVolumeMountRequestSourceCLIDrivenProject,
    ProtoVolumeMountRequestSourceProject,
    ProtoVolumeMountRequestSourceVersionControlProject,
    ProtoVolumeMountRequests,
)
from savvihub.cli.constants import WEB_HOST, PROJECT_TYPE_VERSION_CONTROL, PROJECT_TYPE_CLI_DRIVEN
from savvihub.cli.exceptions import ExitException
from savvihub.cli.formatter import TreeFormatter
from savvihub.cli.inputs.experiment import (
    cluster_name_callback,
    env_vars_callback,
    image_url_callback,
    message_callback,
    start_command_callback,
)
from savvihub.cli.inputs.git import (
    git_branch_callback,
    git_diff_callback,
    git_ref_callback,
)
from savvihub.cli.inputs.organization import organization_name_option
from savvihub.cli.inputs.project import project_name_option
from savvihub.cli.inputs.resource import (
    cpu_limit_callback,
    gpu_limit_callback,
    gpu_type_callback,
    memory_limit_callback,
    processor_type_callback,
    resource_name_callback,
)
from savvihub.cli.inputs.sweep import (
    algorithm_callback,
    max_experiment_count_callback,
    max_failed_experiment_count_callback,
    objective_goal_callback,
    objective_metric_callback,
    objective_type_callback,
    parallel_experiment_count_callback,
    search_space_parameters_callback,
    sweep_config_file_callback
)
from savvihub.cli.inputs.volume import dataset_mount_callback
from savvihub.cli.typer import Context, Typer
from savvihub.common.utils import parse_time_to_ago, short_string

sweep_app = Typer()


@sweep_app.callback()
def main():
    """
    Run a hyperparameter optimization with sweep
    """


@sweep_app.command(user_required=True)
def list(
    ctx: Context,
    organization_name: str = organization_name_option,
    project_name: str = project_name_option,
):
    """
    Display a list of sweeps
    """
    sweeps = ctx.authenticated_client.sweep_list(organization_name, project_name).results
    if not sweeps:
        raise ExitException(f'There is no sweep in `{project_name}` project.')

    table = AsciiTable([
        ['NAME', 'STATUS', 'STATUS_REASON', 'CREATED', 'CREATED_BY', 'EXPERIMENT_COUNT'],
        *[[s.name, s.status, s.status_reason, parse_time_to_ago(s.created_dt), s.created_by.display_name,
           f'{s.experiment_summary.total}/{s.max_experiment_count}']
          for s in sweeps],
    ])
    table.inner_column_border = False
    table.inner_heading_row_border = False
    table.inner_footing_row_border = False
    table.outer_border = False

    typer.echo(table.table)


@sweep_app.command(user_required=True)
def describe(
    ctx: Context,
    organization_name: str = organization_name_option,
    project_name: str = project_name_option,
    sweep_name: str = typer.Argument(..., help="The unique sweep name"),
):
    """
    Describe the sweep in details
    """
    sweep = ctx.authenticated_client.sweep_read(organization_name, project_name, sweep_name)
    timezone = datetime.now().astimezone().tzinfo

    root = TreeFormatter()
    root.add_child(f'Name: {sweep.name}',
                   f'Message: {sweep.message}',
                   f'Created: {sweep.created_dt.astimezone(timezone)}',
                   f'Updated: {sweep.created_dt.astimezone(timezone)}',
                   f'Status: {sweep.status}',
                   f'Status updated: {sweep.status_last_updated.astimezone(timezone)}',
                   f'Status reason: {sweep.status_reason}',
                   f'Organization: {sweep.organization.name}',
                   f'Project: {sweep.project.name}')

    # Objective description
    objective_desc = TreeFormatter('Objective:')
    objective_desc.add_child(f'Type: {sweep.objective.type}',
                             f'Metric: {sweep.objective.metric}',
                             f'Goal: {sweep.objective.goal}')
    root.add_child(objective_desc)

    # Algorithm description
    root.add_child(f'Algorithm: {sweep.algorithm}')

    # Search Space
    if sweep.search_space.parameters:
        search_space_desc = TreeFormatter('Search space:')
        for param in sweep.search_space.parameters:
            search_space_desc.add_child(f'Name: {param.name}')
            search_space_desc.add_child(f'Type: {param.type}')
            range_desc = TreeFormatter('Range:')
            if param.range.max and param.range.min:
                range_desc.add_child(f'Max: {param.range.max}, Min: {param.range.min}')
            if param.range.list:
                if param.range.step:
                    range_desc.add_child(f'List: {param.range.list}, Step: {param.range.step}')
                else:
                    range_desc.add_child(f'List: {param.range.list}')
            search_space_desc.add_child(range_desc)
        root.add_child(search_space_desc)

    # Experiment trials
    root.add_child(f'Max experiment count: {sweep.max_experiment_count}')
    root.add_child(f'Parallel experiment count: {sweep.parallel_experiment_count}')
    root.add_child(f'Max failed experiment count: {sweep.max_failed_experiment_count}')

    # Suggestion histories
    if sweep.suggestion_histories.histories:
        suggestion_desc = TreeFormatter('Suggestions:')
        for history in sweep.suggestion_histories.histories:
            params = []
            for param in history.parameters:
                params.append(f'{param.name}: {param.val}')
            if history.metric_value:
                params.append(f'{sweep.objective.metric}: {history.metric_value}')
            suggestion_desc.add_child(f'{", ".join(params)}')
        root.add_child(suggestion_desc)

    # Sweep histories
    if sweep.histories:
        histories_desc = TreeFormatter('Histories:')
        for history in sweep.histories:
            history_desc = f'- CREATED: {parse_time_to_ago(history.created_dt)}\n'
            history_desc += f'    STATUS: {history.status}\n'
            history_desc += f'    MESSAGE: {history.message}'
            histories_desc.add_child(history_desc)
        root.add_child(histories_desc)

    typer.echo(root.format())


@sweep_app.command(user_required=True)
def create(
    ctx: Context,
    organization_name: str = organization_name_option,
    project_name: str = project_name_option,
    file: str = typer.Option(None, "--file", "-f", callback=sweep_config_file_callback, help="Sweep config file"),
    objective_type: str = typer.Option("", "--objective-type", callback=objective_type_callback,
                                       help="Objective type"),
    objective_goal: float = typer.Option(None, "--objective-goal", callback=objective_goal_callback,
                                         help="Objective goal"),
    objective_metric: str = typer.Option("", "--objective-metric", callback=objective_metric_callback,
                                         help="Objective metric"),
    algorithm: str = typer.Option("", "--algorithm", callback=algorithm_callback, help="Sweep algorithm"),
    search_space_parameters: Optional[List[str]] = typer.Option(None, "--search-space",
                                                                callback=search_space_parameters_callback,
                                                                help="Search space parameters"),
    max_experiment_count: int = typer.Option(None, "--max-experiment-count", callback=max_experiment_count_callback,
                                             help="Maximum number of experiments"),
    parallel_experiment_count: int = typer.Option(None, "--parallel-experiment-count",
                                                  callback=parallel_experiment_count_callback,
                                                  help="Number of experiments to be run in parallel"),
    max_failed_experiment_count: int = typer.Option(None, "--max-failed-experiment-count",
                                                    callback=max_failed_experiment_count_callback,
                                                    help="Maximum number of experiments to allow to fail"),
    message: str = typer.Option("", "--message", "-m", callback=message_callback, help="Experiment message"),
    start_command: str = typer.Option("", "--start-command", callback=start_command_callback, help="Start command"),
    cluster_name: str = typer.Option("", "--cluster", "-c", callback=cluster_name_callback, help="Cluster name"),
    resource_name: str = typer.Option(None, "--resource", "-r", callback=resource_name_callback,
                                      help="Resource name (for savvihub-managed cluster)"),
    processor_type: str = typer.Option(None, "--processor-type", callback=processor_type_callback,
                                       help="cpu or gpu (for custom cluster)"),
    cpu_limit: float = typer.Option(0, "--cpu-limit", callback=cpu_limit_callback,
                                    help="Number of vCPUs (for custom cluster)"),
    memory_limit: str = typer.Option(None, "--memory-limit", callback=memory_limit_callback,
                                     help="Memory capacity (ex: 4Gi, 500Mi)"),
    gpu_type: str = typer.Option(None, "--gpu-type", callback=gpu_type_callback,
                                 help="GPU product name such as Tesla-K80 (for custom cluster)"),
    gpu_limit: int = typer.Option(0, "--gpu-limit", callback=gpu_limit_callback,
                                  help="Number of GPU cores (for custom cluster)"),
    image_url: str = typer.Option("", "--image", "-i", callback=image_url_callback, help="Kernel docker image URL"),
    dataset_mounts: List[str] = typer.Option([], "--dataset", "-d", callback=dataset_mount_callback,
                                             help="Dataset mounted path"),
    ignore_git_diff: bool = typer.Option(False, "--ignore-git-diff", help="Ignore git diff flag"),
    git_branch: str = typer.Option(None, "--git-branch", callback=git_branch_callback, help="Git branch name"),
    git_ref: str = typer.Option(None, "--git-ref", callback=git_ref_callback, help="Git commit SHA"),
    git_diff_arg: str = typer.Option(None, "--git-diff", callback=git_diff_callback, help="Git diff file URL"),
    env_vars: List[str] = typer.Option([], "-e", callback=env_vars_callback, help="Environment variables"),
    output_dir: str = typer.Option("/output/", "--output-dir",
                                   help="A directory to which the experiment result output files to be stored."),
    working_dir: str = typer.Option(None, "--working-dir", help="If not present, use `/work/{project_name}`."),
    root_volume_size: str = typer.Option(None, "--root-volume-size", help="Root volume size"),
):
    """
    Create a sweep
    """
    resource_spec = resource_spec_id = None
    if ctx.store['cluster'].is_savvihub_managed:
        resource_spec_id = ctx.store['resource_spec_id']
    else:
        resource_spec = ModelKernelResourceSpecField(
            processor_type=processor_type,
            cpu_type='Any',
            cpu_limit=cpu_limit,
            memory_limit=memory_limit,
            gpu_type=gpu_type,
            gpu_limit=gpu_limit,
        )

    project_volume_mount = ProtoVolumeMountRequestSourceProject(
        project_id=ctx.project.id,
    )
    project_mount_path = "/work/"
    if ctx.project.type == PROJECT_TYPE_VERSION_CONTROL:
        project_volume_mount.project_type = PROJECT_TYPE_VERSION_CONTROL
        project_volume_mount.version_control_project = ProtoVolumeMountRequestSourceVersionControlProject(
            project_branch=git_branch,
            project_git_ref=git_ref,
            project_git_diff=ctx.store.get('git_diff_uploaded_path')
        )
        project_mount_path += f"{project_name}"
    elif ctx.project.type == PROJECT_TYPE_CLI_DRIVEN:
        project_volume_mount.project_type = PROJECT_TYPE_CLI_DRIVEN
        project_volume_mount.cli_driven_project = ProtoVolumeMountRequestSourceCLIDrivenProject(
            local_project=ctx.store.get('local_project_uploaded_path')
        )

    sweep = ctx.authenticated_client.sweep_create(
        organization=organization_name,
        project=project_name,
        objective=ModelSweepObjective(
            type=ctx.store['objective_type'],
            goal=str(ctx.store['objective_goal']),
            metric=str(ctx.store['objective_metric']),
        ),
        algorithm=ctx.store['algorithm'],
        search_space=ModelSweepSearchSpace(
            parameters=[ModelParameter(
                name=p['name'],
                range=ModelRange(
                    max=p['range'].get('max'),
                    min=p['range'].get('min'),
                    step=p['range'].get('step'),
                    list=[str(elem) for elem in p['range'].get('list', [])],
                ),
                type=p['type'],
            ) for p in ctx.store['search_space']]
        ),
        parallel_experiment_count=ctx.store['parallel_experiment_count'],
        max_experiment_count=ctx.store['max_experiment_count'],
        max_failed_experiment_count=ctx.store['max_failed_experiment_count'],
        message=message,
        cluster_name=ctx.store['cluster'].name,
        image_url=image_url,
        resource_spec_id=resource_spec_id,
        resource_spec=resource_spec,
        start_command=start_command,
        env_vars=ctx.store['env_vars'],
        volume_mounts=ProtoVolumeMountRequests(
            root_volume_size=root_volume_size,
            working_dir=working_dir,
            requests=ctx.store['dataset_mounts'] + [
                ProtoVolumeMountRequest(
                    mount_type='empty-dir',
                    mount_path='/work/'
                ),
                ProtoVolumeMountRequest(
                    mount_type='output',
                    mount_path=output_dir,
                ),
                ProtoVolumeMountRequest(
                    mount_type='project',
                    mount_path=project_mount_path,
                    project=project_volume_mount,
                ),
            ],
        )
    )

    typer.echo(
        f'Sweep {sweep.name} is created. Check the sweep status at below link\n'
        f'{WEB_HOST}/{organization_name}/{project_name}/sweeps/{sweep.name}'
    )


@sweep_app.command(user_required=True)
def stop(
    ctx: Context,
    organization_name: str = organization_name_option,
    project_name: str = project_name_option,
    sweep_name: str = typer.Argument(..., help="The unique sweep name"),
):
    """
    Stop the sweep
    """
    sweep = ctx.authenticated_client.sweep_stop(organization_name, project_name, sweep_name)
    typer.echo(
        f'Sweep {sweep.name} is terminated. Check the sweep status at below link\n'
        f'{WEB_HOST}/{organization_name}/{project_name}/sweeps/{sweep.name}'
    )


@sweep_app.command(user_required=True)
def logs(
    ctx: Context,
    organization_name: str = organization_name_option,
    project_name: str = project_name_option,
    sweep_name: str = typer.Argument(..., help="The unique sweep name"),
    tail: int = typer.Option(200, "--tail"),
    detail: bool = typer.Option(False, "--detail", hidden=True),
    all: bool = typer.Option(False, "--all", hidden=True),
):
    """
    Display the last fifty lines of the sweep logs
    """
    client = ctx.authenticated_client

    kwargs = {}
    if not all:
        kwargs = {'limit': tail}
    if detail:
        kwargs = {'with_event_log': 'true'}

    sweep_logs = client.sweep_logs(organization_name, project_name, sweep_name, **kwargs).logs

    timezone = datetime.now().astimezone().tzinfo
    log_str = ''
    for log in sorted(sweep_logs, key=lambda x: x.timestamp):
        log_str += f'[{datetime.fromtimestamp(log.timestamp, tz=timezone).strftime("%H:%M:%S.%f")}] {log.message}\n'

    typer.echo(log_str)
    typer.echo(
        f'Full logs at:\n'
        f'    {WEB_HOST}/{organization_name}/{project_name}/sweeps/{sweep_name}/logs\n'
    )



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class ProtoVolumeMountRequest(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'dataset': 'ProtoVolumeMountRequestSourceDataset',
        'dataset_version': 'ProtoVolumeMountRequestSourceDatasetVersion',
        'empty_dir': 'object',
        'host_path': 'ProtoVolumeMountRequestSourceHostPath',
        'mount_path': 'str',
        'mount_type': 'str',
        'nfs': 'ProtoVolumeMountRequestSourceNFS',
        'output': 'object',
        'project': 'ProtoVolumeMountRequestSourceProject',
        'readonly': 'bool',
        'volume': 'ProtoVolumeMountRequestSourceVolume'
    }

    attribute_map = {
        'dataset': 'dataset',
        'dataset_version': 'dataset_version',
        'empty_dir': 'empty_dir',
        'host_path': 'host_path',
        'mount_path': 'mount_path',
        'mount_type': 'mount_type',
        'nfs': 'nfs',
        'output': 'output',
        'project': 'project',
        'readonly': 'readonly',
        'volume': 'volume'
    }

    def __init__(self, dataset=None, dataset_version=None, empty_dir=None, host_path=None, mount_path=None, mount_type=None, nfs=None, output=None, project=None, readonly=None, volume=None, local_vars_configuration=None):  # noqa: E501
        """ProtoVolumeMountRequest - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._dataset = None
        self._dataset_version = None
        self._empty_dir = None
        self._host_path = None
        self._mount_path = None
        self._mount_type = None
        self._nfs = None
        self._output = None
        self._project = None
        self._readonly = None
        self._volume = None
        self.discriminator = None

        if dataset is not None:
            self.dataset = dataset
        if dataset_version is not None:
            self.dataset_version = dataset_version
        if empty_dir is not None:
            self.empty_dir = empty_dir
        if host_path is not None:
            self.host_path = host_path
        self.mount_path = mount_path
        self.mount_type = mount_type
        if nfs is not None:
            self.nfs = nfs
        if output is not None:
            self.output = output
        if project is not None:
            self.project = project
        if readonly is not None:
            self.readonly = readonly
        if volume is not None:
            self.volume = volume

    @property
    def dataset(self):
        """Gets the dataset of this ProtoVolumeMountRequest.  # noqa: E501


        :return: The dataset of this ProtoVolumeMountRequest.  # noqa: E501
        :rtype: ProtoVolumeMountRequestSourceDataset
        """
        return self._dataset

    @dataset.setter
    def dataset(self, dataset):
        """Sets the dataset of this ProtoVolumeMountRequest.


        :param dataset: The dataset of this ProtoVolumeMountRequest.  # noqa: E501
        :type dataset: ProtoVolumeMountRequestSourceDataset
        """

        self._dataset = dataset

    @property
    def dataset_version(self):
        """Gets the dataset_version of this ProtoVolumeMountRequest.  # noqa: E501


        :return: The dataset_version of this ProtoVolumeMountRequest.  # noqa: E501
        :rtype: ProtoVolumeMountRequestSourceDatasetVersion
        """
        return self._dataset_version

    @dataset_version.setter
    def dataset_version(self, dataset_version):
        """Sets the dataset_version of this ProtoVolumeMountRequest.


        :param dataset_version: The dataset_version of this ProtoVolumeMountRequest.  # noqa: E501
        :type dataset_version: ProtoVolumeMountRequestSourceDatasetVersion
        """

        self._dataset_version = dataset_version

    @property
    def empty_dir(self):
        """Gets the empty_dir of this ProtoVolumeMountRequest.  # noqa: E501


        :return: The empty_dir of this ProtoVolumeMountRequest.  # noqa: E501
        :rtype: object
        """
        return self._empty_dir

    @empty_dir.setter
    def empty_dir(self, empty_dir):
        """Sets the empty_dir of this ProtoVolumeMountRequest.


        :param empty_dir: The empty_dir of this ProtoVolumeMountRequest.  # noqa: E501
        :type empty_dir: object
        """

        self._empty_dir = empty_dir

    @property
    def host_path(self):
        """Gets the host_path of this ProtoVolumeMountRequest.  # noqa: E501


        :return: The host_path of this ProtoVolumeMountRequest.  # noqa: E501
        :rtype: ProtoVolumeMountRequestSourceHostPath
        """
        return self._host_path

    @host_path.setter
    def host_path(self, host_path):
        """Sets the host_path of this ProtoVolumeMountRequest.


        :param host_path: The host_path of this ProtoVolumeMountRequest.  # noqa: E501
        :type host_path: ProtoVolumeMountRequestSourceHostPath
        """

        self._host_path = host_path

    @property
    def mount_path(self):
        """Gets the mount_path of this ProtoVolumeMountRequest.  # noqa: E501


        :return: The mount_path of this ProtoVolumeMountRequest.  # noqa: E501
        :rtype: str
        """
        return self._mount_path

    @mount_path.setter
    def mount_path(self, mount_path):
        """Sets the mount_path of this ProtoVolumeMountRequest.


        :param mount_path: The mount_path of this ProtoVolumeMountRequest.  # noqa: E501
        :type mount_path: str
        """
        if self.local_vars_configuration.client_side_validation and mount_path is None:  # noqa: E501
            raise ValueError("Invalid value for `mount_path`, must not be `None`")  # noqa: E501

        self._mount_path = mount_path

    @property
    def mount_type(self):
        """Gets the mount_type of this ProtoVolumeMountRequest.  # noqa: E501


        :return: The mount_type of this ProtoVolumeMountRequest.  # noqa: E501
        :rtype: str
        """
        return self._mount_type

    @mount_type.setter
    def mount_type(self, mount_type):
        """Sets the mount_type of this ProtoVolumeMountRequest.


        :param mount_type: The mount_type of this ProtoVolumeMountRequest.  # noqa: E501
        :type mount_type: str
        """
        if self.local_vars_configuration.client_side_validation and mount_type is None:  # noqa: E501
            raise ValueError("Invalid value for `mount_type`, must not be `None`")  # noqa: E501
        allowed_values = ["empty-dir", "project", "dataset", "dataset-version", "output", "volume", "nfs", "host-path"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and mount_type not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `mount_type` ({0}), must be one of {1}"  # noqa: E501
                .format(mount_type, allowed_values)
            )

        self._mount_type = mount_type

    @property
    def nfs(self):
        """Gets the nfs of this ProtoVolumeMountRequest.  # noqa: E501


        :return: The nfs of this ProtoVolumeMountRequest.  # noqa: E501
        :rtype: ProtoVolumeMountRequestSourceNFS
        """
        return self._nfs

    @nfs.setter
    def nfs(self, nfs):
        """Sets the nfs of this ProtoVolumeMountRequest.


        :param nfs: The nfs of this ProtoVolumeMountRequest.  # noqa: E501
        :type nfs: ProtoVolumeMountRequestSourceNFS
        """

        self._nfs = nfs

    @property
    def output(self):
        """Gets the output of this ProtoVolumeMountRequest.  # noqa: E501


        :return: The output of this ProtoVolumeMountRequest.  # noqa: E501
        :rtype: object
        """
        return self._output

    @output.setter
    def output(self, output):
        """Sets the output of this ProtoVolumeMountRequest.


        :param output: The output of this ProtoVolumeMountRequest.  # noqa: E501
        :type output: object
        """

        self._output = output

    @property
    def project(self):
        """Gets the project of this ProtoVolumeMountRequest.  # noqa: E501


        :return: The project of this ProtoVolumeMountRequest.  # noqa: E501
        :rtype: ProtoVolumeMountRequestSourceProject
        """
        return self._project

    @project.setter
    def project(self, project):
        """Sets the project of this ProtoVolumeMountRequest.


        :param project: The project of this ProtoVolumeMountRequest.  # noqa: E501
        :type project: ProtoVolumeMountRequestSourceProject
        """

        self._project = project

    @property
    def readonly(self):
        """Gets the readonly of this ProtoVolumeMountRequest.  # noqa: E501


        :return: The readonly of this ProtoVolumeMountRequest.  # noqa: E501
        :rtype: bool
        """
        return self._readonly

    @readonly.setter
    def readonly(self, readonly):
        """Sets the readonly of this ProtoVolumeMountRequest.


        :param readonly: The readonly of this ProtoVolumeMountRequest.  # noqa: E501
        :type readonly: bool
        """

        self._readonly = readonly

    @property
    def volume(self):
        """Gets the volume of this ProtoVolumeMountRequest.  # noqa: E501


        :return: The volume of this ProtoVolumeMountRequest.  # noqa: E501
        :rtype: ProtoVolumeMountRequestSourceVolume
        """
        return self._volume

    @volume.setter
    def volume(self, volume):
        """Sets the volume of this ProtoVolumeMountRequest.


        :param volume: The volume of this ProtoVolumeMountRequest.  # noqa: E501
        :type volume: ProtoVolumeMountRequestSourceVolume
        """

        self._volume = volume

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ProtoVolumeMountRequest):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ProtoVolumeMountRequest):
            return True

        return self.to_dict() != other.to_dict()



import typer

from savvihub.api.exceptions import NotFoundAPIException
from savvihub.cli.commands.volume import volume_file_list, volume_file_remove, volume_file_copy
from savvihub.cli.exceptions import ExitException
from savvihub.cli.inputs.dataset import dataset_full_name_callback, download_dest_path_callback
from savvihub.cli.inputs.organization import organization_name_option
from savvihub.cli.typer import Typer, Context
from savvihub.cli.utils import parse_dataset, inquire_text

dataset_files_app = Typer()


@dataset_files_app.callback()
def main():
    """
    Manage files in the dataset
    """
    return


@dataset_files_app.command(user_required=True)
def ls(
    ctx: Context,
    dataset_full_name: str = typer.Argument(None, metavar='[DATASET]',
                                            help='{organization}/{dataset} format where organization is optional.',
                                            callback=dataset_full_name_callback),
    path: str = typer.Argument(None, callback=lambda path: path or inquire_text('Path', '/')),
    recursive: bool = typer.Option(False, '-r', '--recursive', help='recursive flag'),
    directory: bool = typer.Option(False, '-d', '--directory', help='list the directory itself, not its contents')
):
    """
    List files in the dataset with prefix
    """
    organization_name, dataset_name, dataset_version_name = parse_dataset(dataset_full_name)
    if organization_name is None:
        organization_name = ctx.organization.name

    dataset = ctx.authenticated_client.dataset_read(organization_name, dataset_name)
    volume_file_list(ctx, dataset.volume_id, path, recursive, directory)


@dataset_files_app.command(user_required=True)
def rm(
    ctx: Context,
    dataset_full_name: str = typer.Argument(None, metavar='[DATASET]',
                                            help='{organization}/{dataset} format where organization is optional.',
                                            callback=dataset_full_name_callback),
    path: str = typer.Argument(None, callback=lambda path: path or inquire_text('Path to delete', '')),
    recursive: bool = typer.Option(False, '-r', '-R', '--recursive',
                                   help='Remove directories and their contents recursively'),
):
    """
    Remove files in a dataset (SavviHub dataset files only)
    """
    organization_name, dataset_name, _ = parse_dataset(dataset_full_name)
    if organization_name is None:
        organization_name = ctx.organization.name

    dataset = ctx.authenticated_client.dataset_read(organization_name, dataset_name)
    volume_file_remove(ctx, dataset.volume_id, path, recursive)


@dataset_files_app.command(user_required=True)
def upload(
    ctx: Context,
    dataset_full_name: str = typer.Argument(None, metavar='[DATASET]',
                                            help='{organization}/{dataset} format where organization is optional.',
                                            callback=dataset_full_name_callback),
    source_path: str = typer.Argument(None, callback=lambda path: path or inquire_text('Source file or directory path', '')),
    dest_path: str = typer.Argument(None, callback=lambda path: path or inquire_text('Dest path', '/')),
    recursive: bool = typer.Option(False, '-r', '--recursive'),
    watch: bool = typer.Option(False, '-w', '--watch'),
):
    """
    Upload files to a dataset (SavviHub dataset only)
    """
    organization_name, dataset_name, _ = parse_dataset(dataset_full_name)
    if organization_name is None:
        organization_name = ctx.organization.name

    dataset = ctx.authenticated_client.dataset_read(organization_name, dataset_name)
    volume_file_copy(
        ctx,
        source_volume_id=None,
        source_path=source_path,
        dest_volume_id=dataset.volume_id,
        dest_path=dest_path,
        recursive=recursive,
        watch=watch,
    )


@dataset_files_app.command(user_required=True)
def download(
    ctx: Context,
    dataset_full_name: str = typer.Argument(None, metavar='DATASET',
                                            help='{organization}/{dataset} format where organization is optional.',
                                            callback=dataset_full_name_callback),
    source_path: str = typer.Argument(None, callback=lambda path: path or inquire_text('Source file or directory path', '/')),
    dest_path: str = typer.Argument(None, callback=download_dest_path_callback),
):
    """
    Download files from a dataset
    """
    organization_name, dataset_name, _ = parse_dataset(dataset_full_name)
    if organization_name is None:
        organization_name = ctx.organization.name

    dataset = ctx.authenticated_client.dataset_read(organization_name, dataset_name)
    volume_file_copy(
        ctx,
        source_volume_id=dataset.volume_id,
        source_path=source_path,
        dest_volume_id=None,
        dest_path=dest_path,
        recursive=True,
        watch=False,
    )


@dataset_files_app.command(user_required=True)
def cp(
    ctx: Context,
    dataset_full_name: str = typer.Argument(None, metavar='DATASET',
                                            help='{organization}/{dataset} format where organization is optional.',
                                            callback=dataset_full_name_callback),
    source_path: str = typer.Argument(None, callback=lambda path: path or inquire_text('Source file or directory path', '')),
    dest_path: str = typer.Argument(None, callback=lambda path: path or inquire_text('Dest path', '')),
    organization_name: str = organization_name_option,
    recursive: bool = typer.Option(False, '-r', '--recursive'),
    watch: bool = typer.Option(False, '-w', '--watch'),
):
    """
    Copy files within a dataset (SavviHub dataset files only)
    """
    organization_name_override, dataset_name, src_dataset_version_name = parse_dataset(dataset_full_name)
    if organization_name_override:
        organization_name = organization_name_override

    dataset = ctx.authenticated_client.dataset_read(organization_name, dataset_name)
    if src_dataset_version_name != 'latest':
        try:
            ctx.authenticated_client.dataset_version_read(dataset.volume_id, src_dataset_version_name)
        except NotFoundAPIException:
            raise ExitException(f'Invalid dataset dataset_version: {dataset_full_name}\n'
                                f'Please check your dataset and dataset_version exist in organization `{organization_name}`.')
    volume_file_copy(
        ctx,
        source_volume_id=dataset.volume_id,
        source_path=source_path,
        dest_volume_id=dataset.volume_id,
        dest_path=dest_path,
        recursive=recursive,
        watch=watch,
    )



class TreeFormatter:
    def __init__(self, text=None):
        self.children = []
        self.text = text
        self.root = self.text is None

    def add_child(self, *children):
        for child in children:
            if isinstance(child, str):
                self.children.append(TreeFormatter(child))
            else:
                self.children.append(child)

    def format(self, to_str=True):
        lines = [] if self.root else [self.text]
        if self.root:
            for child in self.children:
                lines.extend(child.format(False))
        else:
            for child in self.children:
                for line in child.format(False):
                    lines.append('  ' + line)

        if to_str:
            return '\n'.join(lines)

        return lines
